package cn.com.qmth.examcloud.service.examwork.api;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.dao.ExamSetting4CourseRepo;
import cn.com.qmth.examcloud.service.examwork.dao.ExamSetting4OrgRepo;
import cn.com.qmth.examcloud.service.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;
import cn.com.qmth.examcloud.service.examwork.entity.ExamSetting4Course;
import cn.com.qmth.examcloud.service.examwork.entity.ExamSetting4Org;
import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;
import cn.com.qmth.examcloud.service.examwork.enums.ExamType;
import cn.com.qmth.examcloud.service.examwork.service.ExamService;
import cn.com.qmth.examcloud.service.examwork.service.ExamStudentService;
import io.swagger.annotations.ApiOperation;

/**
 * 考试服务API Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}")
public class ExamApi extends ControllerSupport {

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamService examService;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	ExamSetting4CourseRepo examSetting4CourseRepo;

	@Autowired
	ExamSetting4OrgRepo examSetting4OrgRepo;

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@ApiOperation(value = "查询所有考试批次", notes = "分页带查询")
	@GetMapping("/exam/all/{curPage}/{pageSize}")
	public ResponseEntity getAllExam(HttpServletRequest request, @ModelAttribute Exam examCriteria,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		} else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
		return new ResponseEntity(
				examService.getAllExam(examCriteria,
						new PageRequest(curPage, pageSize, new Sort(Direction.DESC, "id"))),
				HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("/exam/all")
	public ResponseEntity getEnableExam(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "根据机构ID和考试类型查询所有有效考试批次", notes = "根据机构ID和考试类型查询所有有效考试批次")
	@GetMapping("/exam/rootOrgId/{orgId}/{examType}")
	public ResponseEntity getEnableExamByRootOrgId(@PathVariable Long orgId,
			@PathVariable String examType) {
		if (orgId == null) {
			return new ResponseEntity(new ErrorMsg("机构ID不能为空"), HttpStatus.OK);
		}
		Exam examCriteria = new Exam();
		examCriteria.setOrgId(orgId);
		if (StringUtils.isNotBlank(examType)) {
			examCriteria.setExamType(ExamType.valueOf(examType));
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试批次(包含有效无效)", notes = "不分页带查询")
	@GetMapping("/exam/all/both")
	public ResponseEntity getAllExam(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setOrgId(accessUser.getRootOrgId());
		}
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "根据名称查询考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("/exam/name")
	public ResponseEntity getEnableExamByName(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setRootOrgId(accessUser.getRootOrgId());
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.findByNameAndRootOrgId(examCriteria.getName(),
				examCriteria.getRootOrgId()), HttpStatus.OK);
	}

	@ApiOperation(value = "按ID查询考试批次", notes = "ID查询")
	@GetMapping("/exam/{id}")
	public ResponseEntity<Exam> getExamById(@PathVariable Long id) {
		return new ResponseEntity<Exam>(examService.getExamById(id), HttpStatus.OK);
	}

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping("/exam")
	public Exam addExam(HttpServletRequest request, @RequestBody Exam exam) {

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "作弊说明内容过大");
		}

		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			exam.setOrgId(accessUser.getOrgId());
			exam.setRootOrgId(accessUser.getRootOrgId());
			exam.setCreateTime(new Date());
			exam.setCanStuDel(true);
			if (!examService.checkExamName(exam)) {
				throw new StatusException("EXAMWORK-001001", "考试名称已存在,请重新填写");
			}
			Exam ret = examService.insertExam(exam, accessUser);
			return ret;
		} else {
			throw new StatusException("EXAMWORK-001002", "accessUser为空");
		}
	}

	@ApiOperation(value = "更新考试批次", notes = "更新")
	@PutMapping("/exam")
	public Exam updateExam(@RequestBody Exam exam) {
		if (!examService.checkExamName(exam)) {
			throw new StatusException("EXAMWORK-001001", "考试名称已存在,请重新填写");
		}

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("EXAMWORK-001002", "作弊说明内容过大");
		}

		Exam ret = examService.saveExam(exam);
		return ret;
	}

	@ApiOperation(value = "按ID删除考试批次", notes = "删除")
	@DeleteMapping("/exam/{id}")
	public ResponseEntity deleteExam(@PathVariable String id) {
		try {
			List<Long> examIds = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
					.collect(Collectors.toList());
			examService.deleteExam(examIds);
			return new ResponseEntity(HttpStatus.OK);
		} catch (Exception e) {
			log.error("删除失败:", e);
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "启用考试", notes = "启用考试")
	@PutMapping("/exam/enable/{ids}")
	public ResponseEntity enableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(true);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "禁用考试", notes = "禁用考试")
	@PutMapping("/exam/disable/{ids}")
	public ResponseEntity disableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(false);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "回填考生不可删除标志", notes = "回填考生不可删除标志")
	@PutMapping("/exam/canNotDel/{id}")
	public ResponseEntity disableUser(@PathVariable Long id) {
		examRepo.canNotDel(id);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考初始化更新考务信息", notes = "网考初始化更新考务信息")
	@PutMapping("/exam/updateExamInfo")
	public ResponseEntity updateExamInfo(@RequestParam Long examId, @RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		examStudentService.updateExamStudentInfo(examStuId);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考更新考务信息", notes = "网考更新考务信息")
	@PutMapping("/exam/updateExamStudent")
	public ResponseEntity updateExamStudent(@RequestParam Long examId,
			@RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		ExamStudent examStudent = examStudentRepo.findOne(examStuId);
		examStudent.setFinished(true);
		examStudentRepo.save(examStudent);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "查询特殊考试设置", notes = "查询特殊考试设置")
	@GetMapping("/exam/querySpecificExamList/{examId}/{curPage}/{pageSize}")
	public ResponseEntity<?> querySpecificExamList(@PathVariable Long examId,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {

		List<ExamSetting4Course> groupList = examSetting4CourseRepo.findByExamId(examId,
				(curPage - 1) * pageSize, pageSize);

		for (ExamSetting4Course specificExam : groupList) {
			List<Long> courseIdList = Lists.newArrayList();
			specificExam.setCourseIdList(courseIdList);

			List<Map<String, Object>> queryList = jdbcTemplate.queryForList(
					"select t.course_id from ecs_e_exam_course_relation t where t.specific_exam_id=?",
					specificExam.getId());
			if (CollectionUtils.isNotEmpty(queryList)) {
				for (Map<String, Object> map : queryList) {
					Long courseId = Long.valueOf(map.get("COURSE_ID").toString());
					courseIdList.add(courseId);
				}
			}
		}

		return new ResponseEntity<List<ExamSetting4Course>>(groupList, HttpStatus.OK);
	}

	@ApiOperation(value = "通过课程ID查询特殊考试设置", notes = "通过课程ID查询特殊考试设置")
	@GetMapping("/exam/querySpecificExamByCourseId/{courseId}")
	public ResponseEntity<?> querySpecificExamByCourseId(@PathVariable Long courseId) {
		List<ExamSetting4Course> groupList = examSetting4CourseRepo
				.queryCourseGroupsByCourseId(courseId);

		for (ExamSetting4Course specificExam : groupList) {
			List<Long> courseIdList = Lists.newArrayList();
			specificExam.setCourseIdList(courseIdList);

			List<Map<String, Object>> queryList = jdbcTemplate.queryForList(
					"select t.course_id from ecs_e_exam_course_relation t where t.specific_exam_id=?",
					specificExam.getId());
			if (CollectionUtils.isNotEmpty(queryList)) {
				for (Map<String, Object> map : queryList) {
					Long curCourseId = Long.valueOf(map.get("COURSE_ID").toString());
					courseIdList.add(curCourseId);
				}
			}
		}

		return new ResponseEntity<Object>(groupList, HttpStatus.OK);
	}

	@ApiOperation(value = "更新特殊考试设置", notes = "更新特殊考试设置")
	@PutMapping("/exam/specificExam")
	public ResponseEntity<?> updateSpecificExam(@RequestBody ExamSetting4Course specificExam,
			HttpServletRequest request) {

		examSetting4CourseRepo.save(specificExam);
		Long specificExamId = specificExam.getId();
		Long examId = specificExam.getExamId();

		List<Long> courseIdList = specificExam.getCourseIdList();
		jdbcTemplate.update("delete from ecs_e_exam_course_relation  where specific_exam_id=?",
				specificExamId);

		if (CollectionUtils.isNotEmpty(courseIdList)) {
			for (Long courseId : courseIdList) {
				jdbcTemplate.update(
						"delete from ecs_e_exam_course_relation  where exam_id=? and course_id=?",
						specificExamId, courseId);
				jdbcTemplate.update(
						"insert into ecs_e_exam_course_relation(specific_exam_id,course_id,exam_id) values(?,?,?)",
						specificExamId, courseId, examId);
			}
		}

		return new ResponseEntity<Object>(HttpStatus.OK);
	}

	@ApiOperation(value = "删除特殊考试设置", notes = "删除特殊考试设置")
	@DeleteMapping("/exam/specificExam/{id}")
	public ResponseEntity<?> deleteSpecificExam(@PathVariable Long id, HttpServletRequest request) {
		examSetting4CourseRepo.delete(id);
		jdbcTemplate.update("delete from ecs_e_exam_course_relation  where group_id=?", id);
		return new ResponseEntity<Object>(HttpStatus.OK);
	}

	@ApiOperation(value = "查询考试相关的学习中心设置", notes = "查询考试相关的学习中心设置")
	@GetMapping("/exam/examSetting4OrgList/{examId}/{curPage}/{pageSize}")
	public List<?> getExamSetting4OrgList(@PathVariable Long examId, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "updateTime");
		List<ExamSetting4Org> allByExamId = examSetting4OrgRepo.findAllByExamId(examId, pageable);
		return allByExamId;
	}

	@ApiOperation(value = "新增考试相关的学习中心设置", notes = "新增考试相关的学习中心设置")
	@PostMapping("/exam/examSetting4Org")
	public ExamSetting4Org addExamSetting4Org(@RequestBody ExamSetting4Org examSetting4Org) {
		return null;
	}

	@ApiOperation(value = "更新考试相关的学习中心设置", notes = "更新考试相关的学习中心设置")
	@PutMapping("/exam/examSetting4Org")
	public ExamSetting4Org updateExamSetting4Org(@RequestBody ExamSetting4Org examSetting4Org) {
		return null;
	}

}
